{ nixpkgs      ? import ./lib/nixpkgs.nix
, lib          ? import (nixpkgs + "/lib")
, pkgsFun      ? import nixpkgs
, hostPlatform ? null         # if left `null` this will be set based on the ./platform/
}:
let
  nixpkgsArgs = { config.allowNonSource = false; };
  nixpkgsOnBuildForBuild  = pkgsFun nixpkgsArgs;

  ownerboot = { overlays }: let
    base = final: prev: {
      inherit hostPlatform;
      nixpkgsOnBuildForHost = pkgsFun
        (nixpkgsArgs
         // (lib.optionalAttrs (final.hostPlatform!=null) {
           crossSystem = final.hostPlatform;
         }));

      coreboot-toolchain = final.callPackage ./coreboot-toolchain { };

      # TODO(amjoseph): move this into a `config` attrset
      console-device = throw "platforms must override this (example: \"ttyS0\")";
      console-parameters = "115200n8";
      linux-command-line =
        let console = "${final.console-device},${final.console-parameters}";
        in "console=${console} earlyprintk=${console}";

      iasl_20180531 = final.nixpkgsOnBuildForBuild.callPackage ./coreboot/iasl_20180531 { };
      coreboot = final.callPackage ./coreboot { };
      image         = final.callPackage ./image { };
      kernel        = final.callPackage ./kernel { };
      userspace     = final.callPackage ./userspace { };
      initramfs     = final.callPackage ./initramfs { };
      flashrom      = final.nixpkgsOnBuildForHost.callPackage ./util/flashrom { };
      main          = final.callPackage ./main { };
      scripts       = prev.scripts or {
        # these are default values to be overridden
        flashrom = throw "platforms must override this attribute";
        flash-write-fallback = throw "platforms must override this attribute";
        flash-write-normal = throw "platforms must override this attribute";
        flash-write-all = throw "platforms must override this attribute";
        nextboot-show = throw "platforms must override this attribute";
        nextboot-use-fallback = throw "platforms must override this attribute";
        nextboot-use-normal = throw "platforms must override this attribute";
        em100-write =
          # currently weakly bound via $PATH because it is often
          # useful to connect them em100 to a machine belonging to a
          # totally different platform than the Device Under Test.
          # TODO(amjoseph): don't hardwire the chip type here
          ''
            em100 -v --stop --holdpin float -c GD25Q128C --download ${final.image}/coreboot.rom --start --trace
          '';
        #
        # TODO(amjoseph): add a sanity check that (a) the image being
        # written to FALLBACK matches the image currently existing in
        # NORMAL and (b) the most recent boot was from NORMAL.  This
        # ensures that the user cannot write an untested image to
        # FALLBACK.  I suppose in theory we also need to verify that
        # NORMAL has not been written to since the most recent boot, but
        # there's not simple way to check that.
      };
    };
  in
    (lib.makeScope
      lib.callPackageWith
      (self: { inherit lib nixpkgsOnBuildForBuild; })).overrideScope'
      (lib.foldl' lib.composeExtensions (_: prev: prev) ([base] ++ overlays));

  common_amd64 = import ./platform/common/amd64.nix { inherit lib; };
  common_arm64 = import ./platform/common/arm64.nix { inherit lib; };

in {
  # one entry for each supported platform
  kevin = ownerboot (import ./platform/kevin { inherit lib common_arm64; });
  am1i  = ownerboot (import ./platform/am1i { inherit lib common_amd64; });
  kgpe  = ownerboot (import ./platform/kgpe { inherit lib common_amd64; });

  em100 = nixpkgsOnBuildForBuild.callPackage ./util/em100 { };
}


